#!/usr/bin/env python

"""
Metadata for document conversion.

Copyright (C) 2018, 2019, 2021, 2023 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.input import get_input
from moinformat.links import get_linker
from moinformat.output import get_output
from moinformat.parsers import get_parser, parsers
from moinformat.serialisers import get_serialiser, serialisers
from moinformat.themes import get_theme

class Metadata:

    "Metadata employed in the document conversion process."

    defaults = {
        "attachments"       : "attachments",
        "input_format"      : "moin",
        "output_context"    : "standalone",
        "output_format"     : "moin",
        "root_pagename"     : "FrontPage",
        }

    default_effects = {
        "output_format"     : "link_format",
        }

    effects = {
        "input_context"     : ["input"],
        "input_format"      : ["parser", "serialiser"],
        "input_separator"   : ["input"],
        "link_format"       : ["linker"],
        "output_context"    : ["output"],
        "output_format"     : ["serialiser"],
        "theme_name"        : ["theme"],
        }

    def __init__(self, parameters=None):

        "Initialise the metadata collection using the 'parameters'."

        self.parameters = parameters or {}

    def __repr__(self):
        return "Metadata(%r)" % self.parameters

    def copy(self):

        "Return a copy of this instance."

        parameters = {}
        parameters.update(self.parameters)
        return self.__class__(parameters)

    def get(self, name, default=None):

        """
        Return the setting for 'name', returning 'default' if 'name' is not
        set. If 'default' is None or omitted and a default is present in the
        defaults registry, this is returned if no setting is defined.
        """

        value = self.parameters.get(name, default)
        if value is None:
            return self.defaults.get(name, default)
        else:
            return value

    def has_key(self, name):
        return self.parameters.has_key(name)

    def set(self, name, value):

        "Set 'name' as 'value' in the metadata."

        self.parameters[name] = value

        # Invalidate any affected settings.

        affected = self.effects.get(name)

        if affected:
            for affected_name in affected:
                if self.has_key(affected_name):
                    del self.parameters[affected_name]

        # Set any default values.

        affected = self.default_effects.get(name)

        if affected and not self.get(affected):
            self.set(affected, value)

    def get_update(self, name, value=None):

        """
        Obtain the 'name' setting, this being overwritten by 'value' if
        specified. Return the updated setting.
        """

        # Overwrite any existing setting.

        if value:
            self.set(name, value)
            return value
        else:
            return self.get(name)

    def make_object(self, name, cls):

        """
        Make an object to be stored in the setting 'name', using 'cls' as the
        object class.
        """

        # Return any existing, preserved object. Since updates to various
        # properties will discard objects, any preserved object should still be
        # applicable.

        obj = self.get(name)
        if obj:
            return obj

        # Without any object class, return None.

        if not cls:
            self.set(name, None)
            return None

        # Instantiate the class and record the object.

        obj = cls(self)
        self.set(name, obj)
        return obj

    def get_input(self, name=None):

        """
        Make an input context using any given 'name' or otherwise using the
        "input_context" setting which will be replaced by any given 'name'.
        """

        cls = get_input(self.get_update("input_context", name))

        return self.make_object("input", cls)

    def get_linker(self, name=None):

        """
        Make a linker using any given 'name' or otherwise using the
        "link_format" setting which will be replaced by any given 'name'.
        """

        cls = get_linker(self.get_update("link_format", name))

        return self.make_object("linker", cls)

    def get_output(self, name=None):

        """
        Make an output context using any given 'name' or otherwise using the
        "output_context" setting which will be replaced by any given 'name'.
        """

        cls = get_output(self.get_update("output_context", name))

        return self.make_object("output", cls)

    def get_parser(self, name=None):

        """
        Make a parser using any given 'name' or otherwise using the
        "input_format" setting which will be replaced by any given 'name'.
        """

        cls = get_parser(self.get_update("input_format", name))

        parser = self.make_object("parser", cls)
        parser.parsers = parsers
        return parser

    def get_serialiser(self, name=None):

        """
        Make a serialiser using any given 'name' or otherwise using the
        "output_format" setting which will be replaced by any given 'name'.
        """

        cls = get_serialiser(self.get_update("output_format", name),
                             self.get("input_format"))

        serialiser = self.make_object("serialiser", cls)
        serialiser.serialisers = serialisers
        return serialiser

    def get_theme(self, name=None):

        """
        Make a theme using any given 'name' or otherwise using the "theme_name"
        setting which will be replaced by any given 'name'.
        """

        cls = get_theme(self.get_update("theme_name", name))

        return self.make_object("theme", cls)

# vim: tabstop=4 expandtab shiftwidth=4
