#!/usr/bin/env python

"""
HTML document fragment parser.

Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.parsers.common import ParserBase
from moinformat.tree.html import Element, Fragment
from moinformat.utils.htmlparse import Parser

class HTMLParser(ParserBase):

    "A prettyprinted document tree parser."

    formats = ["html"]

    def __init__(self, metadata):
        self.metadata = metadata

    def parse(self, s):

        "Parse the tree structure representation in 's'."

        doc = Parser(s).parse()

        # If only a fragment is involved, find the body node and return its
        # children in a fragment.

        if self.metadata.get("fragment") or self.metadata.get("theme_name"):
            body = self._find_body(doc)

            if body:
                return Fragment(body.nodes)
            else:
                return None

        # Otherwise, return the top-level node.

        else:
            return doc

    def _find_body(self, node):

        """
        Find the body element from 'node', returning the element if found or
        None otherwise.
        """

        # Search all nodes with children.

        if isinstance(node, Fragment):

            # Return the node if it is a body element.

            if isinstance(node, Element) and node.name == "body":
                return node

            for n in node.nodes:
                body = self._find_body(n)
                if body:
                    return body

        return None

parser = HTMLParser

# vim: tabstop=4 expandtab shiftwidth=4
